@@ -139,6 +139,13 @@ build_filename (const TrackDetails *track)
 static void
 cleanup (void)
 {
+  /* We're not extracting any more */
+  extracting = FALSE;
+
+  /* Remove any state icons from the model */
+  gtk_list_store_set (track_store, &track->iter,
+                 COLUMN_STATE, STATE_IDLE, -1);
+  
   /* Free the used data */
   if (paths) {
     g_list_deep_free (paths, NULL);
@@ -293,13 +300,13 @@ pop_and_extract (void)
       return;
     }
    
-	/* Update the state stock image */
+    /* Update the state stock image */
     gtk_list_store_set (track_store, &track->iter,
                    COLUMN_STATE, STATE_EXTRACTING, -1);
 		
     /* Update the progress bars */
     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar),
-                                   CLAMP (1.0 - ((g_list_length (pending) + 1)  / (float)total_extracting), 0.0, 1.0));
+                                   CLAMP ((float)current_duration / (float)total_duration, 0.0, 1.0));
 
     /* Now actually do the extraction */
     sj_extractor_extract_track (extractor, track, file_path, &error);
@@ -356,7 +363,7 @@ update_speed_progress (SjExtractor *extractor, float speed, int eta)
   char *eta_str;
 
   if (eta >= 0) {
-    eta_str = g_strdup_printf (_("Estimated time left: %d:%02d (at %0.1fx)"), eta / 60, eta % 60, speed);	
+    eta_str = g_strdup_printf (_("Estimated time left: %d:%02d (at %0.1f\303\227)"), eta / 60, eta % 60, speed);	
   } else {
     eta_str = g_strdup (_("Estimated time left: unknown"));  
   }
@@ -447,7 +454,6 @@ show_finished_dialog (void)
   g_list_foreach (paths, (GFunc)base_finder, &base);
 
   cleanup ();
-  extracting = FALSE;
 
   dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
                                    GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -485,12 +491,15 @@ show_finished_dialog (void)
 static void
 on_completion_cb (SjExtractor *extractor, gpointer data)
 {
-  gtk_list_store_set (track_store, &track->iter,
-                    COLUMN_STATE, STATE_IDLE, -1);
+  /* Only manipulate the track state if we have an album, as we might be here if
+     the disk was ejected mid-rip. */
+  if (gtk_tree_model_iter_n_children (GTK_TREE_MODEL (track_store), NULL) > 0) {
+    /* Remove the track state */
+    gtk_list_store_set (track_store, &track->iter, COLUMN_STATE, STATE_IDLE, -1);
+    /* Uncheck the Extract check box */
+    gtk_list_store_set (track_store, &track->iter, COLUMN_EXTRACT, FALSE, -1);
+  }
   
-  /* Uncheck the Extract check box */
-  gtk_list_store_set (track_store, &track->iter, COLUMN_EXTRACT, FALSE, -1);
-
   if (pending == NULL) {
     show_finished_dialog ();
     if (autostart) {
@@ -512,9 +521,6 @@ on_error_cb (SjExtractor *extractor, GError *error, gpointer data)
 {
   GtkWidget *dialog;
   char *text;
-  /* We've come to a screeching halt... */
-  gtk_list_store_set (track_store, &track->iter,
-                 COLUMN_STATE, STATE_IDLE, -1);
   /* Display a nice dialog */
   text = g_strdup_printf (_("Sound Juicer could not extract this CD.\nReason: %s"), error->message);
 
@@ -529,7 +535,6 @@ on_error_cb (SjExtractor *extractor, GError *error, gpointer data)
 
   /* No need to free the error passed in */
   cleanup ();
-  extracting = FALSE;
 }
 
 /**
@@ -539,10 +544,7 @@ void
 on_progress_cancel_clicked (GtkWidget *button, gpointer user_data)
 {
   sj_extractor_cancel_extract (extractor);
-  gtk_list_store_set (track_store, &track->iter,
-                 COLUMN_STATE, STATE_IDLE, -1);
   cleanup ();
-  extracting = FALSE;
 }
 
 /**
@@ -637,9 +639,9 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
 /**
  * Perform magic on a path to make it safe.
  *
- * This will always replace '/' with ' ', and optionally make the file
- * name shell-friendly. This involves removing [?*\ ] and replacing
- * with '_'.
+ * This will always replace '/' with ' ', and optionally make the file name
+ * shell-friendly. This involves removing [?*\ ] and replacing with '_'.  Also
+ * any leading periods are removed so that the files don't end up being hidden.
  *
  * This function doesn't change the input, and returns an allocated 
  * string.
@@ -648,7 +650,13 @@ static char*
 sanitize_path (const char* str)
 {
   gchar *res = NULL;
-  gchar *s = g_strdup(str);
+  gchar *s;
+
+  /* Skip leading periods, otherwise files disappear... */
+  while (*str == '.')
+    str++;
+  
+  s = g_strdup(str);
   /* Replace path seperators with a hyphen */
   g_strdelimit (s, "/", '-');
   if (strip_chars) {
@@ -672,36 +680,54 @@ sanitize_path (const char* str)
  * Valid markers so far are:
  * %at -- album title
  * %aa -- album artist
+ * %aA -- album artist (lowercase)
+ * %as -- album artist sortname
+ * %aS -- album artist sortname (lowercase)
  * %tn -- track number (i.e 8)
  * %tN -- track number, zero padded (i.e 08)
  * %tt -- track title
  * %ta -- track artist
+ * %tA -- track artist (lowercase)
+ * %ts -- track artist sortname
+ * %tS -- track artist sortname (lowercase)
  */
 char*
 filepath_parse_pattern (const char* pattern, const TrackDetails *track)
 {
   /* p is the pattern iterator, i is a general purpose iterator */
-  const char *p, *i;
-  /* string is the output string, s is the iterator, t is a general string */
-  char *string, *s, *temp;
-  s = string = g_new0 (char, 256); /* TODO: bad hardcoding */
+  const char *p;
+  char *tmp, *str;
+  GString *s;
+
+  if (pattern == NULL || pattern[0] == 0)
+    return g_strdup (" ");
+
+  s = g_string_new (NULL);
 
   p = pattern;
   while (*p) {
-    gchar *tmp; 
+    char *string = NULL;
+    gboolean go_next = TRUE;
+
     /* If not a % marker, copy and continue */
     if (*p != '%') {
-      *s++ = *p++;
+      if ((*p == ' ') && (strip_chars)) {
+        g_string_append_c (s, '_');
+      } else {
+        g_string_append_unichar (s, g_utf8_get_char (p));
+      }
+      p = g_utf8_next_char (p);
       /* Explicit increment as we continue past the increment */
       continue;
     }
+
     /* Is a % marker, go to next and see what to do */
     switch (*++p) {
     case '%':
       /*
        * Literal %
        */
-      *s++ = '%';
+      g_string_append_c (s, '%');
       break;
     case 'a':
       /*
@@ -709,31 +735,37 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
        */
       switch (*++p) {
       case 't':
-        i = temp = sanitize_path (track->album->title);
-        while (*i) *s++ = *i++;
-        g_free (temp);
+        string = sanitize_path (track->album->title);
         break;
       case 'T':
-	tmp = g_utf8_strdown (track->album->title, -1);
-        i = temp = sanitize_path (tmp);
-	g_free(tmp);
-        while (*i) *s++ = *i++;
-        g_free (temp);
+        tmp = g_utf8_strdown (track->album->title, -1);
+        string = sanitize_path (tmp);
+        g_free (tmp);
         break;
       case 'a':
-        i = temp = sanitize_path (track->album->artist);
-        while (*i) *s++ = *i++;
-        g_free (temp);
+        string = sanitize_path (track->album->artist);
         break;
       case 'A':
-	tmp = g_utf8_strdown (track->album->artist, -1);
-        i = temp = sanitize_path (tmp);
-	g_free(tmp);
-        while (*i) *s++ = *i++;
-        g_free (temp);
+        tmp = g_utf8_strdown (track->album->artist, -1);
+        string = sanitize_path (tmp);
+        g_free (tmp);
+        break;
+      case 's':
+        string = sanitize_path (track->album->artist_sortname ? track->album->artist_sortname : track->album->artist);
+        break;
+      case 'S':
+        tmp = g_utf8_strdown (track->album->artist_sortname ? track->album->artist_sortname : track->album->artist, -1);
+        string = sanitize_path (tmp);
+        g_free(tmp);
         break;
       default:
-        *s++ = '%'; *s++ = 'a'; *s++ = *p;
+        /* append "%a", and then the unicode character */
+        g_string_append (s, "%a");
+        p += 2;
+
+        g_string_append_unichar (s, g_utf8_get_char (p));
+        p = g_utf8_next_char (p);
+        go_next = FALSE;
       }
       break;
     case 't':
@@ -742,51 +774,65 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
        */
       switch (*++p) {
       case 't':
-        i = temp = sanitize_path (track->title);
-        while (*i) *s++ = *i++;
-        g_free (temp);
+        string = sanitize_path (track->title);
         break;
       case 'T':
-	tmp = g_utf8_strdown (track->title, -1);
-        i = temp = sanitize_path (tmp);
-	g_free(tmp);
-        while (*i) *s++ = *i++;
-        g_free (temp);
+        tmp = g_utf8_strdown (track->title, -1);
+        string = sanitize_path (tmp);
+        g_free(tmp);
         break;
       case 'a':
-        i = temp = sanitize_path (track->artist);
-        while (*i) *s++ = *i++;
-        g_free (temp);
+        string = sanitize_path (track->artist);
         break;
       case 'A':
-	tmp = g_utf8_strdown (track->artist, -1);
-        i = temp = sanitize_path (tmp);
-	g_free(tmp);
-        while (*i) *s++ = *i++;
-        g_free (temp);
+        tmp = g_utf8_strdown (track->artist, -1);
+        string = sanitize_path (tmp);
+        g_free(tmp);
+        break;
+      case 's':
+        string = sanitize_path (track->artist_sortname ? track->album->artist_sortname : track->artist);
+        break;
+      case 'S':
+        tmp = g_utf8_strdown (track->artist_sortname ? track->album->artist_sortname : track->artist, -1);
+        string = sanitize_path (tmp);
+        g_free(tmp);
         break;
       case 'n':
         /* Track number */
-        i = temp = g_strdup_printf ("%d", track->number);
-        while (*i) *s++ = *i++;
-        g_free (temp);
+        string = g_strdup_printf ("%d", track->number);
         break;
       case 'N':
         /* Track number, zero-padded */
-        i = temp = g_strdup_printf ("%02d", track->number);
-        while (*i) *s++ = *i++;
-        g_free (temp);
+        string = g_strdup_printf ("%02d", track->number);
         break;
       default:
-        *s++ = '%'; *s++ = 't'; *s++ = *p;
+        /* append "%a", and then the unicode character */
+        g_string_append (s, "%t");
+        p += 2;
+
+        g_string_append_unichar (s, g_utf8_get_char (p));
+        p = g_utf8_next_char (p);
+        go_next = FALSE;
       }
       break;
-    default:
-      *s++ = '%';
-      *s++ = *p;
+      default:
+      /* append "%", and then the unicode character */
+      g_string_append_c (s, '%');
+      p += 1;
+
+      g_string_append_unichar (s, g_utf8_get_char (p));
+      p = g_utf8_next_char (p);
     }
-    ++p;
+
+    if (string)
+      g_string_append (s, string);
+    g_free (string);
+
+    if (go_next)
+      ++p;
   }
-  *s = '\0';
-  return string;
+
+  str = s->str;
+  g_string_free (s, FALSE);
+  return str;
 }
